# coding=utf-8

import asyncio, aiomysql, aioredis, json, signal

# # 文章表
# create table zbpblog_arts(
#      id int unsigned not null primary key auto_increment,
#      category varchar(40) not null default '',
#      title varchar(255) not null default '',
#      create_time int,
#      url varchar(100) unique
#  )engine=innodb default charset=utf8mb4;

# # 文章详情表
# create table zbpblog_arts_content(
#      aid int unsigned not null unique,
#      content mediumtext
#  )engine=innodb default charset=utf8mb4;

DB_CONF = {
    'host':'127.0.0.1',
    'port':3306,
    'user':'root',
    'password':'123456',
    'db':'test',
    'charset':'utf8mb4',
    'cursorclass':aiomysql.DictCursor,
    'autocommit':False,
}

REDIS_CONF = {
    "host":'127.0.0.1',
    "port":'6379',
    "password":"123456",
    'db':0
}

class DataSaveHelper:
    stop = False    # 用于停止savw2Db协程

    def __init__(self, key):
        self.key = key

    # 初始化mysql连接池和redis连接
    # aioredis的使用可以参考官方文档 https://aioredis.readthedocs.io/en/v1.3.1/api_reference.html
    async def initConnection(self):
        self.dbpool = await aiomysql.create_pool(**DB_CONF)    # 创建一个mysql连接池, 默认的最大连接数为10
        # self.redis = await aioredis.create_connection((REDIS_CONF['host'], REDIS_CONF['port']), password=REDIS_CONF['password'], db=REDIS_CONF['db'])
        self.redis = await aioredis.create_redis((REDIS_CONF['host'], REDIS_CONF['port']),
                                                      password=REDIS_CONF['password'], db=REDIS_CONF['db'])

    async def save2Db(self):
        # 将数据存入mysql
        async with self.dbpool.acquire() as conn:
            cursor = await conn.cursor()

            sql1 = "insert ignore into zbpblog_arts values (%s, %s, %s, %s, %s)"
            sql2 = "insert ignore into zbpblog_arts_content values (%s, %s)"

            # 从redis中取出item数据
            while not self.stop:
                try:
                    item = await self.redis.brpop(self.key)     # 可能会阻塞
                    data = json.loads(item[1])
                except BaseException as e:
                    print("get zbpblog data failed on line %s: %s" % (e.__traceback__.tb_lineno, e))
                    continue

                art_info = (None, data.get('category', ''), data.get('title', ''), data.get('create_time', ''), data.get('url', ''))

                # 开启事务
                await conn.begin()
                try:
                    await cursor.execute(sql1, art_info)
                    await cursor.execute(sql2, (cursor.lastrowid, data.get('content', '')))
                    await conn.commit()
                except BaseException as e:
                    await conn.rollback()
                    print("Save zbpblog data failed on line %s: %s" % (e.__traceback__.tb_lineno, e))

    # 当接收到终止进程的信号量时(如 Ctrl+C)会触发closeConnection方法
    def closeConnection(self):
        print("Close all the connections")
        self.stop = True
        self.redis.close()
        self.dbpool.close()

        # 关闭事件循环
        # loop = asyncio.get_event_loop()
        # loop.close()
        # exit()

async def main(data_save_helper, coro_num = 10):
    # 连接mysql和redis
    await data_save_helper.initConnection()

    # 创建多个协程来做数据入库的操作
    tasks = [asyncio.ensure_future(data_save_helper.save2Db()) for i in range(coro_num)]
    await asyncio.gather(*tasks)

if __name__ == "__main__":
    coro_num = 3       # 开启10个协程执行数据入mysql库的工作
    key = 'Zbpblog:items'
    data_save_helper = DataSaveHelper(key)     # 创建入库工作的实例
    loop = asyncio.get_event_loop()     # 创建一个事件循环对象

    # 异步的监听结束进程的信号量
    # add_signal_handler方法内部会创建一个协程用于异步接收操作系统向本进程发送的指定信号量
    # 当有任意一个指定信号量被事件循环捕获就会执行closeConnection方法
    for sig in [signal.SIGINT, signal.SIGTERM]:
        print(sig)
        loop.add_signal_handler(sig, data_save_helper.closeConnection)

    loop.run_until_complete(main(data_save_helper))     # 开启事件循环

    # 关闭事件循环
    loop.close()